@@ -2,27 +2,26 @@
 
 #include "StdAfx.h"
 
-#include <stdio.h>
-
-#include "ZipUpdate.h"
-#include "ZipAddCommon.h"
-#include "ZipOut.h"
-
-#include "Common/Defs.h"
 #include "Common/AutoPtr.h"
+#include "Common/Defs.h"
 #include "Common/StringConvert.h"
+
 #include "Windows/Defs.h"
 #include "Windows/Thread.h"
 
+#include "../../Common/CreateCoder.h"
+#include "../../Common/LimitedStreams.h"
+#include "../../Common/OutMemStream.h"
 #include "../../Common/ProgressUtils.h"
 #ifdef COMPRESS_MT
 #include "../../Common/ProgressMt.h"
 #endif
-#include "../../Common/LimitedStreams.h"
-#include "../../Common/OutMemStream.h"
-#include "../../Common/CreateCoder.h"
 
-#include "../../Compress/Copy/CopyCoder.h"
+#include "../../Compress/CopyCoder.h"
+
+#include "ZipAddCommon.h"
+#include "ZipOut.h"
+#include "ZipUpdate.h"
 
 using namespace NWindows;
 using namespace NSynchronization;
@@ -48,8 +47,7 @@ static HRESULT CopyBlockToArchive(ISequentialInStream *inStream,
 {
   CMyComPtr<ISequentialOutStream> outStream;
   outArchive.CreateStreamForCopying(&outStream);
-  CMyComPtr<ICompressCoder> copyCoder = new NCompress::CCopyCoder;
-  return copyCoder->Code(inStream, outStream, NULL, NULL, progress);
+  return NCompress::CopyStream(inStream, outStream, progress);
 }
 
 static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive,
