@@ -11,18 +11,20 @@
 #include "Common/StringConvert.h"
 #include "Common/StringToInt.h"
 
-#include "Windows/FileName.h"
 #include "Windows/FileDir.h"
+#include "Windows/FileName.h"
 #ifdef _WIN32
 #include "Windows/FileMapping.h"
 #include "Windows/Synchronization.h"
 #endif
 
 #include "ArchiveCommandLine.h"
-#include "UpdateAction.h"
-#include "Update.h"
-#include "SortUtils.h"
 #include "EnumDirItems.h"
+#include "SortUtils.h"
+#include "Update.h"
+#include "UpdateAction.h"
+
+#include "myPrivate.h"
 
 extern bool g_CaseSensitive;
 
@@ -32,11 +34,7 @@ extern bool g_CaseSensitive;
 #define MY_isatty_fileno(x) isatty(fileno(x))
 #endif
 
-#define MY_IS_TERMINAL(x) (MY_isatty_fileno(x) != 0); 
-
-#ifdef HAVE_LSTAT
-extern int global_use_lstat;
-#endif
+#define MY_IS_TERMINAL(x) (MY_isatty_fileno(x) != 0);
 
 using namespace NCommandLineParser;
 using namespace NWindows;
@@ -52,7 +50,9 @@ enum Enum
   kDisablePercents,
   kArchiveType,
   kYes,
+  #ifndef _NO_CRYPTO
   kPassword,
+  #endif
   kProperty,
   kOutputDir,
   kWorkingDir,
@@ -72,7 +72,8 @@ enum Enum
   kShowDialog,
   kUseLStat,
   kTechMode,
-  kCaseSensitive
+  kCaseSensitive,
+  kCalcCrc,
 };
 
 }
@@ -82,9 +83,9 @@ static const wchar_t kRecursedIDChar = 'R';
 static const wchar_t *kRecursedPostCharSet = L"0-";
 
 namespace NRecursedPostCharIndex {
-  enum EEnum 
+  enum EEnum
   {
-    kWildCardRecursionOnly = 0, 
+    kWildCardRecursionOnly = 0,
     kNoRecursion = 1
   };
 }
@@ -106,7 +107,7 @@ NExtract::NOverwriteMode::EEnum k_OverwriteModes[] =
   NExtract::NOverwriteMode::kAutoRenameExisting
 };
 
-static const CSwitchForm kSwitchForms[] = 
+static const CSwitchForm kSwitchForms[] =
   {
     { L"?",  NSwitchType::kSimple, false },
     { L"H",  NSwitchType::kSimple, false },
@@ -115,7 +116,9 @@ static const CSwitchForm kSwitchForms[] =
     { L"BD", NSwitchType::kSimple, false },
     { L"T",  NSwitchType::kUnLimitedPostString, false, 1 },
     { L"Y",  NSwitchType::kSimple, false },
+    #ifndef _NO_CRYPTO
     { L"P",  NSwitchType::kUnLimitedPostString, false, 0 },
+    #endif
     { L"M",  NSwitchType::kUnLimitedPostString, true, 1 },
     { L"O",  NSwitchType::kUnLimitedPostString, false, 1 },
     { L"W",  NSwitchType::kUnLimitedPostString, false, 0 },
@@ -135,10 +138,11 @@ static const CSwitchForm kSwitchForms[] =
     { L"AD",  NSwitchType::kSimple, false },
     { L"L",  NSwitchType::kSimple, false },
     { L"SLT", NSwitchType::kSimple, false },
-    { L"SSC", NSwitchType::kPostChar, false, 0, 0, L"-" }
+    { L"SSC", NSwitchType::kPostChar, false, 0, 0, L"-" },
+    { L"SCRC", NSwitchType::kSimple, false }
   };
 
-static const CCommandForm g_CommandForms[] = 
+static const CCommandForm g_CommandForms[] =
 {
   { L"A", false },
   { L"U", false },
@@ -206,7 +210,7 @@ NExtract::NPathMode::EEnum CArchiveCommand::GetPathMode() const
 
 bool CArchiveCommand::IsFromUpdateGroup() const
 {
-  return (CommandType == NCommandType::kAdd || 
+  return (CommandType == NCommandType::kAdd ||
     CommandType == NCommandType::kUpdate ||
     CommandType == NCommandType::kDelete);
 }
@@ -215,9 +219,9 @@ static NRecursedType::EEnum GetRecursedTypeFromIndex(int index)
 {
   switch (index)
   {
-    case NRecursedPostCharIndex::kWildCardRecursionOnly: 
+    case NRecursedPostCharIndex::kWildCardRecursionOnly:
       return NRecursedType::kWildCardOnlyRecursed;
-    case NRecursedPostCharIndex::kNoRecursion: 
+    case NRecursedPostCharIndex::kNoRecursion:
       return NRecursedType::kNonRecursed;
     default:
       return NRecursedType::kRecursed;
@@ -229,7 +233,7 @@ static bool ParseArchiveCommand(const UString &commandString, CArchiveCommand &c
   UString commandStringUpper = commandString;
   commandStringUpper.MakeUpper();
   UString postString;
-  int commandIndex = ParseCommand(kNumCommandForms, g_CommandForms, commandStringUpper, 
+  int commandIndex = ParseCommand(kNumCommandForms, g_CommandForms, commandStringUpper,
       postString) ;
   if (commandIndex < 0)
     return false;
@@ -240,7 +244,7 @@ static bool ParseArchiveCommand(const UString &commandString, CArchiveCommand &c
 // ------------------------------------------------------------------
 // filenames functions
 
-static bool AddNameToCensor(NWildcard::CCensor &wildcardCensor, 
+static bool AddNameToCensor(NWildcard::CCensor &wildcardCensor,
     const UString &name, bool include, NRecursedType::EEnum type)
 {
   bool isWildCard = DoesNameContainWildCard(name);
@@ -262,7 +266,7 @@ static bool AddNameToCensor(NWildcard::CCensor &wildcardCensor,
   return true;
 }
 
-static void AddToCensorFromListFile(NWildcard::CCensor &wildcardCensor, 
+static void AddToCensorFromListFile(NWildcard::CCensor &wildcardCensor,
     LPCWSTR fileName, bool include, NRecursedType::EEnum type, UINT codePage)
 {
   UStringVector names;
@@ -273,7 +277,7 @@ static void AddToCensorFromListFile(NWildcard::CCensor &wildcardCensor,
       throw kIncorrectWildCardInListFile;
 }
 
-static void AddCommandLineWildCardToCensr(NWildcard::CCensor &wildcardCensor, 
+static void AddCommandLineWildCardToCensr(NWildcard::CCensor &wildcardCensor,
     const UString &name, bool include, NRecursedType::EEnum recursedType)
 {
   if (!AddNameToCensor(wildcardCensor, name, include, recursedType))
@@ -282,13 +286,13 @@ static void AddCommandLineWildCardToCensr(NWildcard::CCensor &wildcardCensor,
 
 static void AddToCensorFromNonSwitchesStrings(
     int startIndex,
-    NWildcard::CCensor &wildcardCensor, 
-    const UStringVector &nonSwitchStrings, NRecursedType::EEnum type, 
+    NWildcard::CCensor &wildcardCensor,
+    const UStringVector &nonSwitchStrings, NRecursedType::EEnum type,
     bool thereAreSwitchIncludes, UINT codePage)
 {
-  if(nonSwitchStrings.Size() == startIndex && (!thereAreSwitchIncludes)) 
+  if (nonSwitchStrings.Size() == startIndex && (!thereAreSwitchIncludes))
     AddCommandLineWildCardToCensr(wildcardCensor, kUniversalWildcard, true, type);
-  for(int i = startIndex; i < nonSwitchStrings.Size(); i++)
+  for (int i = startIndex; i < nonSwitchStrings.Size(); i++)
   {
     const UString &s = nonSwitchStrings[i];
     if (s[0] == kFileListID)
@@ -299,8 +303,8 @@ static void AddToCensorFromNonSwitchesStrings(
 }
 
 #ifdef _WIN32
-static void ParseMapWithPaths(NWildcard::CCensor &wildcardCensor, 
-    const UString &switchParam, bool include, 
+static void ParseMapWithPaths(NWildcard::CCensor &wildcardCensor,
+    const UString &switchParam, bool include,
     NRecursedType::EEnum commonRecursedType)
 {
   int splitPos = switchParam.Find(L':');
@@ -337,7 +341,7 @@ static void ParseMapWithPaths(NWildcard::CCensor &wildcardCensor,
         wchar_t c = curData[i];
         if (c == L'\0')
         {
-          AddCommandLineWildCardToCensr(wildcardCensor, 
+          AddCommandLineWildCardToCensr(wildcardCensor,
               name, include, commonRecursedType);
           name.Empty();
         }
@@ -363,11 +367,11 @@ static void ParseMapWithPaths(NWildcard::CCensor &wildcardCensor,
 }
 #endif
 
-static void AddSwitchWildCardsToCensor(NWildcard::CCensor &wildcardCensor, 
-    const UStringVector &strings, bool include, 
+static void AddSwitchWildCardsToCensor(NWildcard::CCensor &wildcardCensor,
+    const UStringVector &strings, bool include,
     NRecursedType::EEnum commonRecursedType, UINT codePage)
 {
-  for(int i = 0; i < strings.Size(); i++)
+  for (int i = 0; i < strings.Size(); i++)
   {
     const UString &name = strings[i];
     NRecursedType::EEnum recursedType;
@@ -408,9 +412,9 @@ static void ConvertToLongName(const UString &prefix, UString &name)
 {
   if (name.IsEmpty() || DoesNameContainWildCard(name))
     return;
-  NFind::CFileInfoW fileInfo;
-  if (NFind::FindFile(prefix + name, fileInfo))
-    name = fileInfo.Name;
+  NFind::CFileInfoW fi;
+  if (fi.Find(prefix + name))
+    name = fi.Name;
 }
 
 static void ConvertToLongNames(const UString &prefix, CObjectVector<NWildcard::CItem> &items)
@@ -451,7 +455,7 @@ static void ConvertToLongNames(const UString &prefix, NWildcard::CCensorNode &no
   for (i = 0; i < node.SubNodes.Size(); i++)
   {
     NWildcard::CCensorNode &nextNode = node.SubNodes[i];
-    ConvertToLongNames(prefix + nextNode.Name + wchar_t(NFile::NName::kDirDelimiter), nextNode); 
+    ConvertToLongNames(prefix + nextNode.Name + wchar_t(NFile::NName::kDirDelimiter), nextNode);
   }
 }
 
@@ -483,14 +487,14 @@ const int kUpdatePairStateNotSupportedActions[] = {2, 2, 1, -1, -1, -1, -1};
 
 const UString kUpdatePairActionIDSet = L"0123"; //Ignore, Copy, Compress, Create Anti
 
-const wchar_t *kUpdateIgnoreItselfPostStringID = L"-"; 
-const wchar_t kUpdateNewArchivePostCharID = '!'; 
+const wchar_t *kUpdateIgnoreItselfPostStringID = L"-";
+const wchar_t kUpdateNewArchivePostCharID = '!';
 
 
-static bool ParseUpdateCommandString2(const UString &command, 
+static bool ParseUpdateCommandString2(const UString &command,
     NUpdateArchive::CActionSet &actionSet, UString &postString)
 {
-  for(int i = 0; i < command.Length();)
+  for (int i = 0; i < command.Length();)
   {
     wchar_t c = MyCharUpper(command[i]);
     int statePos = kUpdatePairStateIDSet.Find(c);
@@ -514,16 +518,16 @@ static bool ParseUpdateCommandString2(const UString &command,
   return true;
 }
 
-static void ParseUpdateCommandString(CUpdateOptions &options, 
-    const UStringVector &updatePostStrings, 
+static void ParseUpdateCommandString(CUpdateOptions &options,
+    const UStringVector &updatePostStrings,
     const NUpdateArchive::CActionSet &defaultActionSet)
 {
-  for(int i = 0; i < updatePostStrings.Size(); i++)
+  for (int i = 0; i < updatePostStrings.Size(); i++)
   {
     const UString &updateString = updatePostStrings[i];
-    if(updateString.CompareNoCase(kUpdateIgnoreItselfPostStringID) == 0)
+    if (updateString.CompareNoCase(kUpdateIgnoreItselfPostStringID) == 0)
     {
-      if(options.UpdateArchiveItself)
+      if (options.UpdateArchiveItself)
       {
         options.UpdateArchiveItself = false;
         options.Commands.Delete(0);
@@ -536,14 +540,14 @@ static void ParseUpdateCommandString(CUpdateOptions &options,
       UString postString;
       if (!ParseUpdateCommandString2(updateString, actionSet, postString))
         ThrowUserErrorException();
-      if(postString.IsEmpty())
+      if (postString.IsEmpty())
       {
-        if(options.UpdateArchiveItself)
+        if (options.UpdateArchiveItself)
           options.Commands[0].ActionSet = actionSet;
       }
       else
       {
-        if(MyCharUpper(postString[0]) != kUpdateNewArchivePostCharID)
+        if (MyCharUpper(postString[0]) != kUpdateNewArchivePostCharID)
           ThrowUserErrorException();
         CUpdateArchiveCommand uc;
         UString archivePath = postString.Mid(1);
@@ -603,20 +607,20 @@ static bool ParseComplexSize(const UString &src, UInt64 &result)
 }
 
 static void SetAddCommandOptions(
-    NCommandType::EEnum commandType, 
-    const CParser &parser, 
+    NCommandType::EEnum commandType,
+    const CParser &parser,
     CUpdateOptions &options)
 {
   NUpdateArchive::CActionSet defaultActionSet;
   switch(commandType)
   {
-    case NCommandType::kAdd: 
+    case NCommandType::kAdd:
       defaultActionSet = NUpdateArchive::kAddActionSet;
       break;
-    case NCommandType::kDelete: 
+    case NCommandType::kDelete:
       defaultActionSet = NUpdateArchive::kDeleteActionSet;
       break;
-    default: 
+    default:
       defaultActionSet = NUpdateArchive::kUpdateActionSet;
   }
   
@@ -626,10 +630,10 @@ static void SetAddCommandOptions(
   CUpdateArchiveCommand updateMainCommand;
   updateMainCommand.ActionSet = defaultActionSet;
   options.Commands.Add(updateMainCommand);
-  if(parser[NKey::kUpdate].ThereIs)
-    ParseUpdateCommandString(options, parser[NKey::kUpdate].PostStrings, 
+  if (parser[NKey::kUpdate].ThereIs)
+    ParseUpdateCommandString(options, parser[NKey::kUpdate].PostStrings,
         defaultActionSet);
-  if(parser[NKey::kWorkingDir].ThereIs)
+  if (parser[NKey::kWorkingDir].ThereIs)
   {
     const UString &postString = parser[NKey::kWorkingDir].PostStrings[0];
     if (postString.IsEmpty())
@@ -659,7 +663,7 @@ static void SetMethodOptions(const CParser &parser, CObjectVector<CProperty> &pr
   if (parser[NKey::kProperty].ThereIs)
   {
     // options.MethodMode.Properties.Clear();
-    for(int i = 0; i < parser[NKey::kProperty].PostStrings.Size(); i++)
+    for (int i = 0; i < parser[NKey::kProperty].PostStrings.Size(); i++)
     {
       CProperty property;
       const UString &postString = parser[NKey::kProperty].PostStrings[i];
@@ -676,7 +680,7 @@ static void SetMethodOptions(const CParser &parser, CObjectVector<CProperty> &pr
   }
 }
 
-CArchiveCommandLineParser::CArchiveCommandLineParser(): 
+CArchiveCommandLineParser::CArchiveCommandLineParser():
   parser(sizeof(kSwitchForms) / sizeof(kSwitchForms[0])) {}
 
 void CArchiveCommandLineParser::Parse1(const UStringVector &commandStrings,
@@ -686,7 +690,7 @@ void CArchiveCommandLineParser::Parse1(const UStringVector &commandStrings,
   {
     parser.ParseStrings(kSwitchForms, commandStrings);
   }
-  catch(...) 
+  catch(...)
   {
     ThrowUserErrorException();
   }
@@ -694,6 +698,7 @@ void CArchiveCommandLineParser::Parse1(const UStringVector &commandStrings,
   options.IsInTerminal = MY_IS_TERMINAL(stdin);
   options.IsStdOutTerminal = MY_IS_TERMINAL(stdout);
   options.IsStdErrTerminal = MY_IS_TERMINAL(stderr);
+  options.StdInMode = parser[NKey::kStdIn].ThereIs;
   options.StdOutMode = parser[NKey::kStdOut].ThereIs;
   options.EnableHeaders = !parser[NKey::kDisableHeaders].ThereIs;
   options.HelpMode = parser[NKey::kHelp1].ThereIs || parser[NKey::kHelp2].ThereIs  || parser[NKey::kHelp3].ThereIs;
@@ -725,13 +730,14 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
 {
   const UStringVector &nonSwitchStrings = parser.NonSwitchStrings;
   int numNonSwitchStrings = nonSwitchStrings.Size();
-  if(numNonSwitchStrings < kMinNonSwitchWords)  
+  if (numNonSwitchStrings < kMinNonSwitchWords)
     ThrowUserErrorException();
 
   if (!ParseArchiveCommand(nonSwitchStrings[kCommandIndex], options.Command))
     ThrowUserErrorException();
 
   options.TechMode = parser[NKey::kTechMode].ThereIs;
+  options.CalcCrc = parser[NKey::kCalcCrc].ThereIs;
 
   if (parser[NKey::kCaseSensitive].ThereIs)
     g_CaseSensitive = (parser[NKey::kCaseSensitive].PostCharIndex < 0);
@@ -748,26 +754,33 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
   if (parser[NKey::kInclude].ThereIs)
   {
     thereAreSwitchIncludes = true;
-    AddSwitchWildCardsToCensor(options.WildcardCensor, 
+    AddSwitchWildCardsToCensor(options.WildcardCensor,
         parser[NKey::kInclude].PostStrings, true, recursedType, codePage);
   }
   if (parser[NKey::kExclude].ThereIs)
-    AddSwitchWildCardsToCensor(options.WildcardCensor, 
+    AddSwitchWildCardsToCensor(options.WildcardCensor,
         parser[NKey::kExclude].PostStrings, false, recursedType, codePage);
  
   int curCommandIndex = kCommandIndex + 1;
-  bool thereIsArchiveName = !parser[NKey::kNoArName].ThereIs && 
-      options.Command.CommandType != NCommandType::kBenchmark && 
+  bool thereIsArchiveName = !parser[NKey::kNoArName].ThereIs &&
+      options.Command.CommandType != NCommandType::kBenchmark &&
       options.Command.CommandType != NCommandType::kInfo;
+
+  bool isExtractGroupCommand = options.Command.IsFromExtractGroup();
+  bool isExtractOrList = isExtractGroupCommand || options.Command.CommandType == NCommandType::kList;
+
+  if (isExtractOrList && options.StdInMode)
+    thereIsArchiveName = false;
+
   if (thereIsArchiveName)
   {
-    if(curCommandIndex >= numNonSwitchStrings)  
+    if (curCommandIndex >= numNonSwitchStrings)
       ThrowUserErrorException();
     options.ArchiveName = nonSwitchStrings[curCommandIndex++];
   }
 
   AddToCensorFromNonSwitchesStrings(
-      curCommandIndex, options.WildcardCensor, 
+      curCommandIndex, options.WildcardCensor,
       nonSwitchStrings, recursedType, thereAreSwitchIncludes, codePage);
 
   options.YesToAll = parser[NKey::kYes].ThereIs;
@@ -776,20 +789,19 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
   global_use_lstat = !parser[NKey::kUseLStat].ThereIs;
 #endif
 
-  bool isExtractGroupCommand = options.Command.IsFromExtractGroup();
-
+  #ifndef _NO_CRYPTO
   options.PasswordEnabled = parser[NKey::kPassword].ThereIs;
-
-  if(options.PasswordEnabled)
+  if (options.PasswordEnabled)
     options.Password = parser[NKey::kPassword].PostStrings[0];
+  #endif
 
-  options.StdInMode = parser[NKey::kStdIn].ThereIs;
   options.ShowDialog = parser[NKey::kShowDialog].ThereIs;
 
-  if(isExtractGroupCommand || options.Command.CommandType == NCommandType::kList)
+  if (parser[NKey::kArchiveType].ThereIs)
+    options.ArcType = parser[NKey::kArchiveType].PostStrings[0];
+
+  if (isExtractOrList)
   {
-    if (options.StdInMode)
-      ThrowException("Reading archives from stdin is not implemented");
     if (!options.WildcardCensor.AllAreRelative())
       ThrowException("Cannot use absolute pathnames for this command");
 
@@ -797,11 +809,11 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
 
     if (parser[NKey::kArInclude].ThereIs)
     {
-      AddSwitchWildCardsToCensor(archiveWildcardCensor, 
+      AddSwitchWildCardsToCensor(archiveWildcardCensor,
         parser[NKey::kArInclude].PostStrings, true, NRecursedType::kNonRecursed, codePage);
     }
     if (parser[NKey::kArExclude].ThereIs)
-      AddSwitchWildCardsToCensor(archiveWildcardCensor, 
+      AddSwitchWildCardsToCensor(archiveWildcardCensor,
       parser[NKey::kArExclude].PostStrings, false, NRecursedType::kNonRecursed, codePage);
 
     bool directlyAddArchiveName = false;
@@ -810,7 +822,7 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
         // no wildcard => no need to scan
         directlyAddArchiveName = true;
       } else {
-        AddCommandLineWildCardToCensr(archiveWildcardCensor, options.ArchiveName, true, NRecursedType::kNonRecursed);
+      AddCommandLineWildCardToCensr(archiveWildcardCensor, options.ArchiveName, true, NRecursedType::kNonRecursed);
       }
     }
 
@@ -820,18 +832,33 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
 
     archiveWildcardCensor.ExtendExclude();
 
-    CObjectVector<CDirItem> dirItems;
+    if (options.StdInMode)
     {
-      UStringVector errorPaths;
-      CRecordVector<DWORD> errorCodes;
-      HRESULT res = EnumerateItems(archiveWildcardCensor, dirItems, NULL, errorPaths, errorCodes);
-      if (res != S_OK || errorPaths.Size() > 0)
-        throw "cannot find archive";
+      UString arcName = parser[NKey::kStdIn].PostStrings.Front();
+      options.ArchivePathsSorted.Add(arcName);
+      options.ArchivePathsFullSorted.Add(arcName);
     }
+    else
+    {
+
     UStringVector archivePaths;
-    int i;
-    for (i = 0; i < dirItems.Size(); i++)
-      archivePaths.Add(dirItems[i].FullPath);
+
+    {
+      CDirItems dirItems;
+      {
+        UStringVector errorPaths;
+        CRecordVector<DWORD> errorCodes;
+        HRESULT res = EnumerateItems(archiveWildcardCensor, dirItems, NULL, errorPaths, errorCodes);
+        if (res != S_OK || errorPaths.Size() > 0)
+          throw "cannot find archive";
+      }
+      for (int i = 0; i < dirItems.Items.Size(); i++)
+      {
+        const CDirItem &dirItem = dirItems.Items[i];
+        if (!dirItem.IsDir())
+          archivePaths.Add(dirItems.GetPhyPath(i));
+      }
+    }
 
     // Because the pathname of archive can be a symbolic link
     // do not use "AddCommandLineWildCardToCensr(archiveWildcardCensor, options.ArchiveName"
@@ -843,6 +870,7 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
 
     UStringVector archivePathsFull;
 
+    int i;
     for (i = 0; i < archivePaths.Size(); i++)
     {
       UString fullPath;
@@ -858,43 +886,42 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
       options.ArchivePathsSorted.Add(archivePaths[indices[i]]);
       options.ArchivePathsFullSorted.Add(archivePathsFull[indices[i]]);
     }
+    
+    }
 
     if (isExtractGroupCommand)
     {
-      SetMethodOptions(parser, options.ExtractProperties); 
+      SetMethodOptions(parser, options.ExtractProperties);
       if (options.StdOutMode && options.IsStdOutTerminal && options.IsStdErrTerminal)
         throw kSameTerminalError;
-      if(parser[NKey::kOutputDir].ThereIs)
+      if (parser[NKey::kOutputDir].ThereIs)
       {
         options.OutputDir = parser[NKey::kOutputDir].PostStrings[0];
         NFile::NName::NormalizeDirPathPrefix(options.OutputDir);
       }
 
       options.OverwriteMode = NExtract::NOverwriteMode::kAskBefore;
-      if(parser[NKey::kOverwrite].ThereIs)
-        options.OverwriteMode = 
+      if (parser[NKey::kOverwrite].ThereIs)
+        options.OverwriteMode =
             k_OverwriteModes[parser[NKey::kOverwrite].PostCharIndex];
       else if (options.YesToAll)
         options.OverwriteMode = NExtract::NOverwriteMode::kWithoutPrompt;
     }
   }
-  else if(options.Command.IsFromUpdateGroup())
+  else if (options.Command.IsFromUpdateGroup())
   {
     CUpdateOptions &updateOptions = options.UpdateOptions;
 
-    if(parser[NKey::kArchiveType].ThereIs)
-      options.ArcType = parser[NKey::kArchiveType].PostStrings[0];
-
-    SetAddCommandOptions(options.Command.CommandType, parser, updateOptions); 
+    SetAddCommandOptions(options.Command.CommandType, parser, updateOptions);
     
-    SetMethodOptions(parser, updateOptions.MethodMode.Properties); 
+    SetMethodOptions(parser, updateOptions.MethodMode.Properties);
 
     options.EnablePercents = !parser[NKey::kDisablePercents].ThereIs;
 
     if (options.EnablePercents)
     {
-      if ((options.StdOutMode && !options.IsStdErrTerminal) || 
-         (!options.StdOutMode && !options.IsStdOutTerminal))  
+      if ((options.StdOutMode && !options.IsStdErrTerminal) ||
+         (!options.StdOutMode && !options.IsStdOutTerminal))
         options.EnablePercents = false;
     }
 
@@ -917,19 +944,19 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
       throw "stdout mode and email mode cannot be combined";
     if (updateOptions.StdOutMode && options.IsStdOutTerminal)
       throw kTerminalOutError;
-    if(updateOptions.StdInMode)
+    if (updateOptions.StdInMode)
       updateOptions.StdInFileName = parser[NKey::kStdIn].PostStrings.Front();
 
     #ifdef _WIN32
     ConvertToLongNames(options.WildcardCensor);
     #endif
   }
-  else if(options.Command.CommandType == NCommandType::kBenchmark)
+  else if (options.Command.CommandType == NCommandType::kBenchmark)
   {
     options.NumThreads = (UInt32)-1;
     options.DictionarySize = (UInt32)-1;
     options.NumIterations = 1;
-    if (curCommandIndex < numNonSwitchStrings)  
+    if (curCommandIndex < numNonSwitchStrings)
     {
       if (!ConvertStringToUInt32(nonSwitchStrings[curCommandIndex++], options.NumIterations))
         ThrowUserErrorException();
@@ -971,10 +998,10 @@ void CArchiveCommandLineParser::Parse2(CArchiveCommandLineOptions &options)
         ThrowUserErrorException();
     }
   }
-  else if(options.Command.CommandType == NCommandType::kInfo)
+  else if (options.Command.CommandType == NCommandType::kInfo)
   {
   }
-  else 
+  else
     ThrowUserErrorException();
   options.WildcardCensor.ExtendExclude();
 }
