@@ -5,48 +5,44 @@
 #include "Common/MyInitGuid.h"
 
 #include "Common/CommandLineParser.h"
-#include "Common/MyException.h"
 #include "Common/IntToString.h"
-#include "Common/ListFileUtils.h"
-#include "Common/StdInStream.h"
+#include "Common/MyException.h"
 #include "Common/StdOutStream.h"
 #include "Common/StringConvert.h"
 #include "Common/StringToInt.h"
-#include "Common/Wildcard.h"
 
-#include "Windows/FileDir.h"
-#include "Windows/FileName.h"
 #include "Windows/Defs.h"
 #include "Windows/Error.h"
+#include "Windows/FileDir.h"
+#include "Windows/FileName.h"
 #ifdef _WIN32
 #include "Windows/MemoryLock.h"
 #endif
 
-#include "../../IPassword.h"
 #include "../../ICoder.h"
-#include "../Common/UpdateAction.h"
-#include "../Common/Update.h"
-#include "../Common/Extract.h"
+#include "../../IPassword.h"
+
 #include "../Common/ArchiveCommandLine.h"
 #include "../Common/ExitCode.h"
+#include "../Common/Extract.h"
 #ifdef EXTERNAL_CODECS
 #include "../Common/LoadCodecs.h"
 #endif
+#include "../Common/PropIDUtils.h"
+#include "../Common/Update.h"
+#include "../Common/UpdateAction.h"
 
 #include "../../Compress/LZMA_Alone/LzmaBenchCon.h"
 
+#include "ExtractCallbackConsole.h"
 #include "List.h"
 #include "OpenCallbackConsole.h"
-#include "ExtractCallbackConsole.h"
 #include "UpdateCallbackConsole.h"
 
 #include "../../MyVersion.h"
 
 #if defined( _WIN32) && defined( _7ZIP_LARGE_PAGES)
-extern "C" 
-{ 
 #include "../../../../C/Alloc.h"
-}
 #endif
 
 #include "myPrivate.h"
@@ -56,7 +52,7 @@ using namespace NWindows;
 using namespace NFile;
 using namespace NCommandLineParser;
 
-HINSTANCE g_hInstance = 0;
+// HINSTANCE g_hInstance = 0;
 extern CStdOutStream *g_StdStream;
 
 static const char *kCopyrightString = "\n7-Zip"
@@ -71,7 +67,7 @@ static const char *kCopyrightString = "\n7-Zip"
 " " MY_VERSION_COPYRIGHT_DATE "\n"
 "p7zip Version " P7ZIP_VERSION ;
 
-static const char *kHelpString = 
+static const char *kHelpString =
     "\nUsage: 7z"
 #ifdef _NO_CRYPTO
     "r"
@@ -108,7 +104,9 @@ static const char *kHelpString =
     "            (ex:  ln -s .. ldir)\n"
 #endif
     "  -o{Directory}: set Output directory\n"
+    #ifndef _NO_CRYPTO
     "  -p{Password}: set Password\n"
+    #endif
     "  -r[-|0]: Recurse subdirectories\n"
     "  (CAUTION: this flag does not do what you think, avoid using it)\n"
     "  -sfx[{name}]: Create SFX archive\n"
@@ -124,9 +122,9 @@ static const char *kHelpString =
         " a dummy -tgzip -so Doc.txt > archive.gz)\n"
     "  -ssc[-]: set sensitive case mode\n"
     "  -t{Type}: Set type of archive\n"
-    "  -v{Size}[b|k|m|g]: Create volumes\n"
     "  -u[-][p#][q#][r#][x#][y#][z#][!newArchiveName]: Update options\n"
-    "  -w[path]: assign Work directory. Empty path means a temporary directory\n"
+    "  -v{Size}[b|k|m|g]: Create volumes\n"
+    "  -w[{path}]: assign Work directory. Empty path means a temporary directory\n"
     "  -x[r[-|0]]]{@listfile|!wildcard}: eXclude filenames\n"
     "  -y: assume Yes on all queries\n";
 
@@ -135,6 +133,7 @@ static const char *kHelpString =
 
 static const char *kEverythingIsOk = "Everything is Ok";
 static const char *kUserErrorMessage  = "Incorrect command line"; // NExitCode::kUserError
+static const char *kNoFormats = "7-Zip cannot find the code that works with archives.";
 
 static const wchar_t *kDefaultSfxModule = L"7zCon.sfx";
 
@@ -174,7 +173,7 @@ static void ShowCopyrightAndHelp(CStdOutStream &s, bool needHelp)
     if (nbcpu > 1) s << nbcpu << " CPUs)\n";
     else           s << nbcpu << " CPU)\n";
 
-  if (needHelp) 
+  if (needHelp)
     s << kHelpString;
 }
 
@@ -201,27 +200,28 @@ static inline char GetHex(Byte value)
   return (char)((value < 10) ? ('0' + value) : ('A' + (value - 10)));
 }
 
+const char *kUnsupportedArcTypeMessage = "Unsupported archive type";
+
 int Main2(
-  #ifndef _WIN32  
+  #ifndef _WIN32
   int numArguments, const char *arguments[]
   #endif
 )
 {
-  #ifdef _WIN32  
+  #ifdef _WIN32
   SetFileApisToOEM();
   #endif
   
   UStringVector commandStrings;
-  #ifdef _WIN32  
+  #ifdef _WIN32
   NCommandLineParser::SplitCommandLine(GetCommandLineW(), commandStrings);
   #else
   // GetArguments(numArguments, arguments, commandStrings);
   extern void mySplitCommandLine(int numArguments,const char *arguments[],UStringVector &parts);
   mySplitCommandLine(numArguments,arguments,commandStrings);
-
   #endif
 
-  if(commandStrings.Size() == 1)
+  if (commandStrings.Size() == 1)
   {
     ShowCopyrightAndHelp(g_StdOut, true);
     return 0;
@@ -234,7 +234,7 @@ int Main2(
 
   parser.Parse1(commandStrings, options);
 
-  if(options.HelpMode)
+  if (options.HelpMode)
   {
     ShowCopyrightAndHelp(g_StdOut, true);
     return 0;
@@ -269,6 +269,17 @@ int Main2(
     throw CSystemException(result);
 
   bool isExtractGroupCommand = options.Command.IsFromExtractGroup();
+
+  if (codecs->Formats.Size() == 0 &&
+        (isExtractGroupCommand ||
+        options.Command.CommandType == NCommandType::kList ||
+        options.Command.IsFromUpdateGroup()))
+    throw kNoFormats;
+
+  CIntVector formatIndices;
+  if (!codecs->FindFormatForArchiveType(options.ArcType, formatIndices))
+    throw kUnsupportedArcTypeMessage;
+
   if (options.Command.CommandType == NCommandType::kInfo)
   {
     stdStream << endl << "Formats:" << endl;
@@ -279,8 +290,8 @@ int Main2(
       #ifdef EXTERNAL_CODECS
       if (arc.LibIndex >= 0)
       {
-        char s[32];
-        ConvertUInt64ToString(arc.LibIndex, s);
+        char s[16];
+        ConvertUInt32ToString(arc.LibIndex, s);
         PrintString(stdStream, s, 2);
       }
       else
@@ -327,15 +338,15 @@ int Main2(
     stdStream << endl << "Codecs:" << endl;
 
     #ifdef EXTERNAL_CODECS
-    UINT32 numMethods;
+    UInt32 numMethods;
     if (codecs->GetNumberOfMethods(&numMethods) == S_OK)
     for (UInt32 j = 0; j < numMethods; j++)
     {
       int libIndex = codecs->GetCodecLibIndex(j);
       if (libIndex >= 0)
       {
-        char s[32];
-        ConvertUInt64ToString(libIndex, s);
+        char s[16];
+        ConvertUInt32ToString(libIndex, s);
         PrintString(stdStream, s, 2);
       }
       else
@@ -396,28 +407,37 @@ int Main2(
   }
   else if (isExtractGroupCommand || options.Command.CommandType == NCommandType::kList)
   {
-    if(isExtractGroupCommand)
+    if (isExtractGroupCommand)
     {
       CExtractCallbackConsole *ecs = new CExtractCallbackConsole;
       CMyComPtr<IFolderArchiveExtractCallback> extractCallback = ecs;
 
       ecs->OutStream = &stdStream;
+
+      #ifndef _NO_CRYPTO
       ecs->PasswordIsDefined = options.PasswordEnabled;
       ecs->Password = options.Password;
+      #endif
+
       ecs->Init();
 
       COpenCallbackConsole openCallback;
       openCallback.OutStream = &stdStream;
+
+      #ifndef _NO_CRYPTO
       openCallback.PasswordIsDefined = options.PasswordEnabled;
       openCallback.Password = options.Password;
+      #endif
 
       CExtractOptions eo;
+      eo.StdInMode = options.StdInMode;
       eo.StdOutMode = options.StdOutMode;
       eo.PathMode = options.Command.GetPathMode();
       eo.TestMode = options.Command.IsTestMode();
       eo.OverwriteMode = options.OverwriteMode;
       eo.OutputDir = options.OutputDir;
       eo.YesToAll = options.YesToAll;
+      eo.CalcCrc = options.CalcCrc;
       #ifdef COMPRESS_MT
       eo.Properties = options.ExtractProperties;
       #endif
@@ -425,9 +445,10 @@ int Main2(
       CDecompressStat stat;
       HRESULT result = DecompressArchives(
           codecs,
-          options.ArchivePathsSorted, 
+          formatIndices,
+          options.ArchivePathsSorted,
           options.ArchivePathsFullSorted,
-          options.WildcardCensor.Pairs.Front().Head, 
+          options.WildcardCensor.Pairs.Front().Head,
           eo, &openCallback, ecs, errorMessage, stat);
       if (!errorMessage.IsEmpty())
       {
@@ -436,15 +457,9 @@ int Main2(
           result = E_FAIL;
       }
 
-      stdStream << endl << endl << "Total:" << endl;
+      stdStream << endl;
       if (ecs->NumArchives > 1)
         stdStream << "Archives: " << ecs->NumArchives << endl;
-      {
-        stdStream << "Folders: " << stat.NumFolders << endl;
-        stdStream << "Files: " << stat.NumFiles << endl;
-        stdStream << "Size: " << stat.UnpackSize << endl;
-        stdStream << "Compressed: " << stat.PackSize << endl;
-      }
       if (ecs->NumArchiveErrors != 0 || ecs->NumFileErrors != 0)
       {
         if (ecs->NumArchives > 1)
@@ -461,19 +476,37 @@ int Main2(
       }
       if (result != S_OK)
         throw CSystemException(result);
+      if (stat.NumFolders != 0)
+        stdStream << "Folders: " << stat.NumFolders << endl;
+      if (stat.NumFiles != 1 || stat.NumFolders != 0)
+          stdStream << "Files: " << stat.NumFiles << endl;
+      stdStream
+           << "Size:       " << stat.UnpackSize << endl
+           << "Compressed: " << stat.PackSize << endl;
+      if (options.CalcCrc)
+      {
+        wchar_t s[16];
+        ConvertUInt32ToHex(stat.CrcSum, s);
+        stdStream << "CRC: " << s << endl;
+      }
     }
     else
     {
       UInt64 numErrors = 0;
       HRESULT result = ListArchives(
           codecs,
-          options.ArchivePathsSorted, 
+          formatIndices,
+          options.StdInMode,
+          options.ArchivePathsSorted,
           options.ArchivePathsFullSorted,
-          options.WildcardCensor.Pairs.Front().Head, 
-          options.EnableHeaders, 
+          options.WildcardCensor.Pairs.Front().Head,
+          options.EnableHeaders,
           options.TechMode,
-          options.PasswordEnabled, 
-          options.Password, numErrors);
+          #ifndef _NO_CRYPTO
+          options.PasswordEnabled,
+          options.Password,
+          #endif
+          numErrors);
       if (numErrors > 0)
       {
         g_StdOut << endl << "Errors: " << numErrors;
@@ -483,36 +516,39 @@ int Main2(
         throw CSystemException(result);
     }
   }
-  else if(options.Command.IsFromUpdateGroup())
+  else if (options.Command.IsFromUpdateGroup())
   {
-    UString workingDir;
-
     CUpdateOptions &uo = options.UpdateOptions;
     if (uo.SfxMode && uo.SfxModule.IsEmpty())
       uo.SfxModule = kDefaultSfxModule;
 
-    bool passwordIsDefined = 
-        options.PasswordEnabled && !options.Password.IsEmpty();
-
     COpenCallbackConsole openCallback;
     openCallback.OutStream = &stdStream;
+
+    #ifndef _NO_CRYPTO
+    bool passwordIsDefined =
+        options.PasswordEnabled && !options.Password.IsEmpty();
     openCallback.PasswordIsDefined = passwordIsDefined;
     openCallback.Password = options.Password;
+    #endif
 
     CUpdateCallbackConsole callback;
     callback.EnablePercents = options.EnablePercents;
+
+    #ifndef _NO_CRYPTO
     callback.PasswordIsDefined = passwordIsDefined;
     callback.AskPassword = options.PasswordEnabled && options.Password.IsEmpty();
     callback.Password = options.Password;
+    #endif
     callback.StdOutMode = uo.StdOutMode;
     callback.Init(&stdStream);
 
     CUpdateErrorInfo errorInfo;
 
-    if (!uo.Init(codecs, options.ArchiveName, options.ArcType))
-      throw "Unsupported archive type";
-    HRESULT result = UpdateArchive(codecs, 
-        options.WildcardCensor, uo, 
+    if (!uo.Init(codecs, formatIndices, options.ArchiveName))
+      throw kUnsupportedArcTypeMessage;
+    HRESULT result = UpdateArchive(codecs,
+        options.WildcardCensor, uo,
         errorInfo, &openCallback, &callback);
 
 #ifdef ENV_UNIX
@@ -600,7 +636,7 @@ int Main2(
     }
     return exitCode;
   }
-  else 
+  else
     PrintHelpAndExit(stdStream);
   return 0;
 }
